/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.ppa;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.kemet.impl.grammar.exceptions.KemetAPIException;
import org.kemet.impl.grammar.integration.HieroglyphManager;
import org.kemet.message.Messages;
import org.kemet.tests.TestRoot;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>tests about PPA.</b>
 * </p>
 * 
 * @since 0.6
 * 
 */
public class TestsPPA extends TestRoot {
	static final Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>initialize the parameters before to execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	public void init() {
		// nothing
	}

	/**
	 * <p>
	 * <b>test if it's the indicator stating of PPA.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test1() {
		// initialize the data tests
		sequence = "M17G43";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>there is no subject.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test2() {
		// initialize the data tests
		sequence = "T28D21";
		result = new String[] { "?" };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_FORMAT);

		String[] res;
		try {
			res = hieroglyph.analyze();
			
			assertEquals(res[0], result[0]);
			log.info("res(i) = " + res[0]);
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>there is a subject.</b>
	 * 
	 * indicator of PPA + name + preposition + name + feminine grammatical kind +
	 * determiner for specify a ideogram .
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test3() {
		// initialize the data tests
		sequence = "M17G43G29V31A1D2Z1N35O4X1M1";
		result = new String[] { Messages.getString("SUBJECT"),
				Messages.getString("PREDICATE") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_FORMAT);

		String[] res;
		try {
			res = hieroglyph.analyze();

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>there is a subject.</b>
	 * 
	 * indicator of PPA + name + preposition + name + feminine grammatical kind +
	 * determiner for specify a ideogram .
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test4() {
		// initialize the data tests
		sequence = "M17G43M22M22O34N35";
		result = new String[] { Messages.getString("SUBJECT") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_FORMAT);

		String[] res;
		try {
			res = hieroglyph.analyze();
			
			assertEquals(res[0], result[0]);
			log.info("res(i) = " + res[0]);
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}
	
	/**
	 * <p>
	 * <b>verify if the proposition is correct.</b>
	 * 
	 * indicator of PPA + name + preposition + name + feminine grammatical kind +
	 * determiner for specify a ideogram .
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test5() {
		// initialize the data tests
		sequence = "M17G43M22M22O34N35";
		hieroglyph = new HieroglyphManager(sequence, gm, phInfo);

		boolean res;
		try {
			res = hieroglyph.propositionIsCorrect(PROPOSITION_TYPE_PPA);			
			assertEquals(res, true);
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}
}
