/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.xml;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.Test;
import org.kemet.tests.IAttr;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>phonograms Tests : test if there are double id ==> PAY ATTENTION : it is
 * can be normal to have several times the same id : this test is made to verify
 * it.</b>
 * </p>
 * 
 * @since 0.1
 * 
 */
public class PhonogramsTests extends TestCase implements IAttr {
	static Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>current XPath.</b>
	 * </p>
	 */
	private XPath xpath;

	/**
	 * <p>
	 * <b>current DOM document.</b>
	 * </p>
	 */
	private Document document;

	/**
	 * <p>
	 * <b>current list of xpath expressions.</b>
	 * </p>
	 */
	private String[] expression;

	/**
	 * <p>
	 * <b>set the parameters before execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Before
	public void setUp() throws Exception {
		// parse the XML as a W3C Document
		DocumentBuilder builder = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder();
		document = builder.parse(new File(XMLPath + "phonogram.xml"));

		xpath = XPathFactory.newInstance().newXPath();
	}

	/**
	 * <p>
	 * <b>test : verify if there are double id with differents types of
	 * phonogram.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testDoubleID() {
		NodeList node;
		Node id;

		try {
			expression = new String[] {
					"/PhonogramCollection/Phonograms/Phonogram/SignList", "id" };

			node = (NodeList) xpath.evaluate(expression[0], document,
					XPathConstants.NODESET);

			XPath xpath = XPathFactory.newInstance().newXPath();

			ArrayList sortList = new ArrayList();

			for (int i = 0; i < node.getLength(); i++) {
				String attr = node.item(i).getParentNode().getAttributes()
						.getNamedItem("phonogramType").getNodeValue();

				id = (Node) xpath.evaluate(expression[1], node.item(i),
						XPathConstants.NODE);

				// store the id with the type of phonogram
				sortList.add(id.getFirstChild().getNodeValue() + "_" + attr);
			}

			Collections.sort(sortList);

			// verify if <id> is present several times...
			ArrayList doub = new ArrayList();
			String previousValue = "";

			for (int i = 0; i < sortList.size(); i++) {
				if (!sortList.get(i).equals(previousValue))
					previousValue = sortList.get(i).toString();
				else
					doub.add(sortList.get(i));
			}

			// display the collection...
			if (doub.size() > 0) {
				log.info("<id> count : " + doub.size());

				for (int i = 0; i < doub.size(); i++)
					log.info(doub.get(i));

				assertEquals(doub.size(), 0);
			}
		} catch (XPathExpressionException e) {
			assertTrue(false);
		}
	}
}
